@@ -129,10 +129,6 @@ class InvalidTemplateSection(HeatException):
     msg_fmt = _("The template section is invalid: %(section)s")
 
 
-class InvalidTemplateParameter(HeatException):
-    msg_fmt = _("The Parameter (%(key)s) has no attributes.")
-
-
 class ImmutableParameterModified(HeatException):
     msg_fmt = _("The following parameters are immutable and may not be "
                 "updated: %(keys)s")
@@ -143,6 +139,11 @@ class ImmutableParameterModified(HeatException):
         super(ImmutableParameterModified, self).__init__(**kwargs)
 
 
+class InvalidMergeStrategyForParam(HeatException):
+    msg_fmt = _("Invalid merge strategy %(strategy)s for "
+                "parameter %(param)s.")
+
+
 class InvalidTemplateAttribute(HeatException):
     msg_fmt = _("The Referenced Attribute (%(resource)s %(key)s)"
                 " is incorrect.")
@@ -153,6 +154,15 @@ class InvalidTemplateReference(HeatException):
                 ' is incorrect.')
 
 
+class TemplateOutputError(HeatException):
+    msg_fmt = _('Error in %(resource)s output %(attribute)s: %(message)s')
+
+
+class InvalidExternalResourceDependency(HeatException):
+    msg_fmt = _("Invalid dependency with external %(resource_type)s "
+                "resource: %(external_id)s")
+
+
 class EntityNotFound(HeatException):
     msg_fmt = _("The %(entity)s (%(name)s) could not be found.")
 
@@ -163,6 +173,10 @@ class EntityNotFound(HeatException):
                                              **kwargs)
 
 
+class PhysicalResourceExists(HeatException):
+    msg_fmt = _("The physical resource for (%(name)s) exists.")
+
+
 class PhysicalResourceNameAmbiguity(HeatException):
     msg_fmt = _(
         "Multiple physical resources were found with name (%(name)s).")
@@ -259,7 +273,7 @@ class ClientNotAvailable(HeatException):
 
 
 class WatchRuleNotFound(EntityNotFound):
-    """Keep this for AWS compatiblility."""
+    """Keep this for AWS compatibility."""
     msg_fmt = _("The Watch Rule (%(watch_name)s) could not be found.")
 
 
@@ -355,7 +369,7 @@ class ResourcePropertyValueDependency(HeatException):
 
 class PropertyUnspecifiedError(HeatException):
     msg_fmt = _('At least one of the following properties '
-                'must be specified: %(props)s')
+                'must be specified: %(props)s.')
 
     def __init__(self, *args, **kwargs):
         if args:
@@ -501,5 +515,10 @@ class SIGHUPInterrupt(HeatException):
     msg_fmt = _("System SIGHUP signal received.")
 
 
-class NoActionRequired(Exception):
-    pass
+class InvalidServiceVersion(HeatException):
+    msg_fmt = _("Invalid service %(service)s version %(version)s")
+
+
+class InvalidTemplateVersions(HeatException):
+    msg_fmt = _('A template version alias %(version)s was added for a '
+                'template class that has no official YYYY-MM-DD version.')
