@@ -21,6 +21,7 @@ from heat.common.i18n import _
 from heat.common.i18n import _LE
 from heat.common import param_utils
 from heat.common import template_format
+from heat.common import timeutils as heat_timeutils
 from heat.engine import constraints as constr
 from heat.rpc import api as rpc_api
 
@@ -133,8 +134,7 @@ def translate_filters(params):
         rpc_api.STACK_TIMEOUT: 'timeout',
         rpc_api.STACK_OWNER: 'username',
         rpc_api.STACK_PARENT: 'owner_id',
-        rpc_api.STACK_USER_PROJECT_ID: 'stack_user_project_id',
-        rpc_api.RES_PHYSICAL_ID: 'nova_instance'
+        rpc_api.STACK_USER_PROJECT_ID: 'stack_user_project_id'
     }
 
     for key, field in key_map.items():
@@ -171,37 +171,33 @@ def translate_filters(params):
     return params
 
 
-def format_stack_outputs(stack, outputs, resolve_value=False):
+def format_stack_outputs(outputs, resolve_value=False):
     """Return a representation of the given output template.
 
     Return a representation of the given output template for the given stack
     that matches the API output expectations.
     """
-    return [format_stack_output(stack, outputs,
-                                key, resolve_value=resolve_value)
+    return [format_stack_output(outputs[key], resolve_value=resolve_value)
             for key in outputs]
 
 
-def format_stack_output(stack, outputs, k, resolve_value=True):
+def format_stack_output(output_defn, resolve_value=True):
     result = {
-        rpc_api.OUTPUT_KEY: k,
-        rpc_api.OUTPUT_DESCRIPTION: outputs[k].get('Description',
-                                                   'No description given'),
+        rpc_api.OUTPUT_KEY: output_defn.name,
+        rpc_api.OUTPUT_DESCRIPTION: output_defn.description(),
     }
 
     if resolve_value:
+        value = None
         try:
-            value = stack.output(k)
+            value = output_defn.get_value()
         except Exception as ex:
             # We don't need error raising, just adding output_error to
             # resulting dict.
-            value = None
             result.update({rpc_api.OUTPUT_ERROR: six.text_type(ex)})
         finally:
             result.update({rpc_api.OUTPUT_VALUE: value})
 
-    if outputs[k].get('error_msg'):
-        result.update({rpc_api.OUTPUT_ERROR: outputs[k].get('error_msg')})
     return result
 
 
@@ -211,14 +207,17 @@ def format_stack(stack, preview=False, resolve_outputs=True):
     Return a representation of the given stack that matches the API output
     expectations.
     """
-    updated_time = stack.updated_time and stack.updated_time.isoformat()
-    created_time = stack.created_time or timeutils.utcnow()
+    updated_time = heat_timeutils.isotime(stack.updated_time)
+    created_time = heat_timeutils.isotime(stack.created_time or
+                                          timeutils.utcnow())
+    deleted_time = heat_timeutils.isotime(stack.deleted_time)
     info = {
         rpc_api.STACK_NAME: stack.name,
         rpc_api.STACK_ID: dict(stack.identifier()),
-        rpc_api.STACK_CREATION_TIME: created_time.isoformat(),
+        rpc_api.STACK_CREATION_TIME: created_time,
         rpc_api.STACK_UPDATED_TIME: updated_time,
-        rpc_api.STACK_NOTIFICATION_TOPICS: [],  # TODO(?) Not implemented yet
+        rpc_api.STACK_DELETION_TIME: deleted_time,
+        rpc_api.STACK_NOTIFICATION_TOPICS: [],  # TODO(therve) Not implemented
         rpc_api.STACK_PARAMETERS: stack.parameters.map(six.text_type),
         rpc_api.STACK_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
         rpc_api.STACK_TMPL_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
@@ -241,13 +240,44 @@ def format_stack(stack, preview=False, resolve_outputs=True):
 
     # allow users to view the outputs of stacks
     if stack.action != stack.DELETE and resolve_outputs:
-        info[rpc_api.STACK_OUTPUTS] = format_stack_outputs(stack,
-                                                           stack.outputs,
+        info[rpc_api.STACK_OUTPUTS] = format_stack_outputs(stack.outputs,
                                                            resolve_value=True)
 
     return info
 
 
+def format_stack_db_object(stack):
+    """Return a summary representation of the given stack.
+
+    Given a stack versioned db object, return a representation of the given
+    stack for a stack listing.
+    """
+    updated_time = heat_timeutils.isotime(stack.updated_at)
+    created_time = heat_timeutils.isotime(stack.created_at)
+    deleted_time = heat_timeutils.isotime(stack.deleted_at)
+
+    tags = None
+    if stack.tags:
+        tags = [t.tag for t in stack.tags]
+    info = {
+        rpc_api.STACK_ID: dict(stack.identifier()),
+        rpc_api.STACK_NAME: stack.name,
+        rpc_api.STACK_DESCRIPTION: '',
+        rpc_api.STACK_ACTION: stack.action,
+        rpc_api.STACK_STATUS: stack.status,
+        rpc_api.STACK_STATUS_DATA: stack.status_reason,
+        rpc_api.STACK_CREATION_TIME: created_time,
+        rpc_api.STACK_UPDATED_TIME: updated_time,
+        rpc_api.STACK_DELETION_TIME: deleted_time,
+        rpc_api.STACK_OWNER: stack.username,
+        rpc_api.STACK_PARENT: stack.owner_id,
+        rpc_api.STACK_USER_PROJECT_ID: stack.stack_user_project_id,
+        rpc_api.STACK_TAGS: tags,
+    }
+
+    return info
+
+
 def format_resource_attributes(resource, with_attr=None):
     resolver = resource.attributes
     if not with_attr:
@@ -262,7 +292,7 @@ def format_resource_attributes(resource, with_attr=None):
     # including the ones are not represented in response of show API, such as
     # 'console_urls' for nova server, user can view it by taking with_attr
     # parameter
-    if 'show' in six.iterkeys(resolver):
+    if 'show' in resolver:
         show_attr = resolve('show', resolver)
         # check if 'show' resolved to dictionary. so it's not None
         if isinstance(show_attr, collections.Mapping):
@@ -274,7 +304,7 @@ def format_resource_attributes(resource, with_attr=None):
             # remove 'show' attribute if it's None or not a mapping
             # then resolve all attributes manually
             del resolver._attributes['show']
-    attributes = set(list(six.iterkeys(resolver)) + with_attr)
+    attributes = set(resolver) | set(with_attr)
     return dict((attr, resolve(attr, resolver))
                 for attr in attributes)
 
@@ -287,7 +317,7 @@ def format_resource_properties(resource):
             return None
 
     return dict((prop, get_property(prop))
-                for prop in six.iterkeys(resource.properties_schema))
+                for prop in resource.properties_schema.keys())
 
 
 def format_stack_resource(resource, detail=True, with_props=False,
@@ -297,9 +327,9 @@ def format_stack_resource(resource, detail=True, with_props=False,
     Return a representation of the given resource that matches the API output
     expectations.
     """
-    created_time = resource.created_time and resource.created_time.isoformat()
-    last_updated_time = (resource.updated_time and
-                         resource.updated_time.isoformat()) or created_time
+    created_time = heat_timeutils.isotime(resource.created_time)
+    last_updated_time = heat_timeutils.isotime(
+        resource.updated_time or resource.created_time)
     res = {
         rpc_api.RES_UPDATED_TIME: last_updated_time,
         rpc_api.RES_CREATION_TIME: created_time,
@@ -316,8 +346,7 @@ def format_stack_resource(resource, detail=True, with_props=False,
     }
 
     if resource.has_nested():
-        res[rpc_api.RES_NESTED_STACK_ID] = dict(
-            resource.nested().identifier())
+        res[rpc_api.RES_NESTED_STACK_ID] = dict(resource.nested_identifier())
 
     if resource.stack.parent_resource_name:
         res[rpc_api.RES_PARENT_RESOURCE] = resource.stack.parent_resource_name
@@ -349,23 +378,22 @@ def format_stack_preview(stack):
     return fmt_stack
 
 
-def format_event(event):
-    stack_identifier = event.stack.identifier()
-    event_timestamp = event.timestamp or timeutils.utcnow()
-
+def format_event(event, stack_identifier, root_stack_identifier=None):
     result = {
-        rpc_api.EVENT_ID: dict(event.identifier()),
+        rpc_api.EVENT_ID: dict(event.identifier(stack_identifier)),
         rpc_api.EVENT_STACK_ID: dict(stack_identifier),
         rpc_api.EVENT_STACK_NAME: stack_identifier.stack_name,
-        rpc_api.EVENT_TIMESTAMP: event_timestamp.isoformat(),
+        rpc_api.EVENT_TIMESTAMP: heat_timeutils.isotime(event.created_at),
         rpc_api.EVENT_RES_NAME: event.resource_name,
         rpc_api.EVENT_RES_PHYSICAL_ID: event.physical_resource_id,
-        rpc_api.EVENT_RES_ACTION: event.action,
-        rpc_api.EVENT_RES_STATUS: event.status,
-        rpc_api.EVENT_RES_STATUS_DATA: event.reason,
+        rpc_api.EVENT_RES_ACTION: event.resource_action,
+        rpc_api.EVENT_RES_STATUS: event.resource_status,
+        rpc_api.EVENT_RES_STATUS_DATA: event.resource_status_reason,
         rpc_api.EVENT_RES_TYPE: event.resource_type,
         rpc_api.EVENT_RES_PROPERTIES: event.resource_properties,
     }
+    if root_stack_identifier:
+        result[rpc_api.EVENT_ROOT_STACK_ID] = dict(root_stack_identifier)
 
     return result
 
@@ -379,32 +407,38 @@ def format_notification_body(stack):
         state = '_'.join(stack.state)
     else:
         state = 'Unknown'
+
+    updated_at = heat_timeutils.isotime(stack.updated_time)
     result = {
         rpc_api.NOTIFY_TENANT_ID: stack.context.tenant_id,
-        rpc_api.NOTIFY_USER_ID: stack.context.user,
+        rpc_api.NOTIFY_USER_ID: stack.context.username,
         # deprecated: please use rpc_api.NOTIFY_USERID for user id or
         # rpc_api.NOTIFY_USERNAME for user name.
         rpc_api.NOTIFY_USERID: stack.context.user_id,
-        rpc_api.NOTIFY_USERNAME: stack.context.user,
+        rpc_api.NOTIFY_USERNAME: stack.context.username,
         rpc_api.NOTIFY_STACK_ID: stack.id,
         rpc_api.NOTIFY_STACK_NAME: stack.name,
         rpc_api.NOTIFY_STATE: state,
         rpc_api.NOTIFY_STATE_REASON: stack.status_reason,
-        rpc_api.NOTIFY_CREATE_AT: stack.created_time.isoformat(),
+        rpc_api.NOTIFY_CREATE_AT: heat_timeutils.isotime(stack.created_time),
+        rpc_api.NOTIFY_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
+        rpc_api.NOTIFY_TAGS: stack.tags,
+        rpc_api.NOTIFY_UPDATE_AT: updated_at
     }
     return result
 
 
 def format_watch(watch):
 
-    updated_at = watch.updated_at or timeutils.utcnow()
+    updated_time = heat_timeutils.isotime(watch.updated_at or
+                                          timeutils.utcnow())
     result = {
         rpc_api.WATCH_ACTIONS_ENABLED: watch.rule.get(
             rpc_api.RULE_ACTIONS_ENABLED),
         rpc_api.WATCH_ALARM_ACTIONS: watch.rule.get(
             rpc_api.RULE_ALARM_ACTIONS),
         rpc_api.WATCH_TOPIC: watch.rule.get(rpc_api.RULE_TOPIC),
-        rpc_api.WATCH_UPDATED_TIME: updated_at.isoformat(),
+        rpc_api.WATCH_UPDATED_TIME: updated_time,
         rpc_api.WATCH_DESCRIPTION: watch.rule.get(rpc_api.RULE_DESCRIPTION),
         rpc_api.WATCH_NAME: watch.name,
         rpc_api.WATCH_COMPARISON: watch.rule.get(rpc_api.RULE_COMPARISON),
@@ -420,8 +454,9 @@ def format_watch(watch):
         rpc_api.WATCH_STATE_REASON: watch.rule.get(rpc_api.RULE_STATE_REASON),
         rpc_api.WATCH_STATE_REASON_DATA:
         watch.rule.get(rpc_api.RULE_STATE_REASON_DATA),
-        rpc_api.WATCH_STATE_UPDATED_TIME: watch.rule.get(
-            rpc_api.RULE_STATE_UPDATED_TIME, timeutils.utcnow()).isoformat(),
+        rpc_api.WATCH_STATE_UPDATED_TIME: heat_timeutils.isotime(
+            watch.rule.get(rpc_api.RULE_STATE_UPDATED_TIME,
+                           timeutils.utcnow())),
         rpc_api.WATCH_STATE_VALUE: watch.state,
         rpc_api.WATCH_STATISTIC: watch.rule.get(rpc_api.RULE_STATISTIC),
         rpc_api.WATCH_THRESHOLD: watch.rule.get(rpc_api.RULE_THRESHOLD),
@@ -432,7 +467,7 @@ def format_watch(watch):
     return result
 
 
-def format_watch_data(wd):
+def format_watch_data(wd, rule_names):
 
     # Demangle DB format data into something more easily used in the API
     # We are expecting a dict with exactly two items, Namespace and
@@ -446,9 +481,9 @@ def format_watch_data(wd):
         return
 
     result = {
-        rpc_api.WATCH_DATA_ALARM: wd.watch_rule.name,
+        rpc_api.WATCH_DATA_ALARM: rule_names.get(wd.watch_rule_id),
         rpc_api.WATCH_DATA_METRIC: metric_name,
-        rpc_api.WATCH_DATA_TIME: wd.created_at.isoformat(),
+        rpc_api.WATCH_DATA_TIME: heat_timeutils.isotime(wd.created_at),
         rpc_api.WATCH_DATA_NAMESPACE: namespace,
         rpc_api.WATCH_DATA: metric_data
     }
@@ -532,7 +567,8 @@ def format_software_config(sc, detail=True):
         rpc_api.SOFTWARE_CONFIG_ID: sc.id,
         rpc_api.SOFTWARE_CONFIG_NAME: sc.name,
         rpc_api.SOFTWARE_CONFIG_GROUP: sc.group,
-        rpc_api.SOFTWARE_CONFIG_CREATION_TIME: sc.created_at.isoformat()
+        rpc_api.SOFTWARE_CONFIG_CREATION_TIME:
+            heat_timeutils.isotime(sc.created_at)
     }
     if detail:
         result[rpc_api.SOFTWARE_CONFIG_CONFIG] = sc.config['config']
@@ -554,11 +590,12 @@ def format_software_deployment(sd):
         rpc_api.SOFTWARE_DEPLOYMENT_STATUS: sd.status,
         rpc_api.SOFTWARE_DEPLOYMENT_STATUS_REASON: sd.status_reason,
         rpc_api.SOFTWARE_DEPLOYMENT_CONFIG_ID: sd.config.id,
-        rpc_api.SOFTWARE_DEPLOYMENT_CREATION_TIME: sd.created_at.isoformat(),
+        rpc_api.SOFTWARE_DEPLOYMENT_CREATION_TIME:
+            heat_timeutils.isotime(sd.created_at),
     }
     if sd.updated_at:
         result[rpc_api.SOFTWARE_DEPLOYMENT_UPDATED_TIME] = (
-            sd.updated_at.isoformat())
+            heat_timeutils.isotime(sd.updated_at))
     return result
 
 
@@ -571,6 +608,14 @@ def format_snapshot(snapshot):
         rpc_api.SNAPSHOT_STATUS: snapshot.status,
         rpc_api.SNAPSHOT_STATUS_REASON: snapshot.status_reason,
         rpc_api.SNAPSHOT_DATA: snapshot.data,
-        rpc_api.SNAPSHOT_CREATION_TIME: snapshot.created_at.isoformat(),
+        rpc_api.SNAPSHOT_CREATION_TIME:
+            heat_timeutils.isotime(snapshot.created_at),
     }
     return result
+
+
+def build_resource_description(docstring):
+    if docstring is not None:
+        return '\n'.join(map(lambda x: x.strip(), docstring.split('\n')))
+    else:
+        return _('No description given')
