@@ -30,19 +30,12 @@ class NeutronClientPlugin(client_plugin.ClientPlugin):
     def _create(self):
 
         con = self.context
-
-        endpoint_type = self._get_client_option('neutron', 'endpoint_type')
-        endpoint = self.url_for(service_type=self.NETWORK,
-                                endpoint_type=endpoint_type)
-
+        interface = self._get_client_option('neutron', 'endpoint_type')
         args = {
-            'auth_url': con.auth_url,
+            'session': con.keystone_session,
             'service_type': self.NETWORK,
-            'token': self.auth_token,
-            'endpoint_url': endpoint,
-            'endpoint_type': endpoint_type,
-            'ca_cert': self._get_client_option('neutron', 'ca_file'),
-            'insecure': self._get_client_option('neutron', 'insecure')
+            'interface': interface,
+            'region_name': self._get_region_name()
         }
 
         return nc.Client(**args)
@@ -100,12 +93,6 @@ class NeutronClientPlugin(client_plugin.ClientPlugin):
                                                                props.pop(key))
         return props[id_key]
 
-    def resolve_loadbalancer(self, props, lb_key, lb_id_key):
-        return self._resolve(props, lb_key, lb_id_key, 'loadbalancer')
-
-    def resolve_listener(self, props, listener_key, listener_id_key):
-        return self._resolve(props, listener_key, listener_id_key, 'listener')
-
     def resolve_pool(self, props, pool_key, pool_id_key):
         if props.get(pool_key):
             props[pool_id_key] = self.find_resourceid_by_name_or_id(
@@ -113,18 +100,9 @@ class NeutronClientPlugin(client_plugin.ClientPlugin):
             props.pop(pool_key)
         return props[pool_id_key]
 
-    def resolve_network(self, props, net_key, net_id_key):
-        return self._resolve(props, net_key, net_id_key, 'network')
-
-    def resolve_subnet(self, props, subnet_key, subnet_id_key):
-        return self._resolve(props, subnet_key, subnet_id_key, 'subnet')
-
     def resolve_router(self, props, router_key, router_id_key):
         return self._resolve(props, router_key, router_id_key, 'router')
 
-    def resolve_port(self, props, port_key, port_id_key):
-        return self._resolve(props, port_key, port_id_key, 'port')
-
     def network_id_from_subnet_id(self, subnet_id):
         subnet_info = self.client().show_subnet(subnet_id)
         return subnet_info['subnet']['network_id']
@@ -177,3 +155,38 @@ class NeutronClientPlugin(client_plugin.ClientPlugin):
                     else:
                         raise exception.PhysicalResourceNameAmbiguity(name=sg)
         return seclist
+
+    def _resove_resource_path(self, resource):
+        """Returns sfc resource path."""
+
+        if resource == 'port_pair':
+            RESOURCE_PATH = "/sfc/port_pairs"
+            return RESOURCE_PATH
+
+    def create_sfc_resource(self, resource, props):
+        """Returns created sfc resource record."""
+
+        path = self._resove_resource_path(resource)
+        record = self.client().create_ext(path, {resource: props}
+                                          ).get(resource)
+        return record
+
+    def update_sfc_resource(self, resource, prop_diff, resource_id):
+        """Returns updated sfc resource record."""
+
+        path = self._resove_resource_path(resource)
+        return self.client().update_ext(path + '/%s', self.resource_id,
+                                        {resource: prop_diff})
+
+    def delete_sfc_resource(self, resource, resource_id):
+        """deletes sfc resource record and returns status"""
+
+        path = self._resove_resource_path(resource)
+        return self.client().delete_ext(path + '/%s', self.resource_id)
+
+    def show_sfc_resource(self, resource, resource_id):
+        """returns specific sfc resource record"""
+
+        path = self._resove_resource_path(resource)
+        return self.client().show_ext(path + '/%s', self.resource_id
+                                      ).get(resource)
