@@ -24,7 +24,6 @@ from heat.common import timeutils
 from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import function
-from heat.engine.hot import template
 from heat.engine import properties
 from heat.engine.resources import stack_resource
 from heat.engine import rsrc_defn
@@ -44,7 +43,9 @@ class ResourceGroup(stack_resource.StackResource):
     for a single resource in the group, synthetic attributes of the form
     `resource.{resource index}.{attribute name}` can be used. The resource ID
     of a particular resource in the group can be obtained via the synthetic
-    attribute `resource.{resource index}`.
+    attribute `resource.{resource index}`. Note, that if you get attribute
+    without `{resource index}`, e.g. `[resource, {attribute_name}]`, you'll get
+    a list of this attribute's value for all resources in group.
 
     While each resource in the group will be identically configured, this
     resource does allow for some index-based customization of the properties
@@ -108,9 +109,9 @@ class ResourceGroup(stack_resource.StackResource):
     )
 
     ATTRIBUTES = (
-        REFS, ATTR_ATTRIBUTES,
+        REFS, REFS_MAP, ATTR_ATTRIBUTES, REMOVED_RSRC_LIST
     ) = (
-        'refs', 'attributes',
+        'refs', 'refs_map', 'attributes', 'removed_rsrc_list'
     )
 
     properties_schema = {
@@ -199,6 +200,12 @@ class ResourceGroup(stack_resource.StackResource):
             _("A list of resource IDs for the resources in the group."),
             type=attributes.Schema.LIST
         ),
+        REFS_MAP: attributes.Schema(
+            _("A map of resource names to IDs for the resources in "
+              "the group."),
+            type=attributes.Schema.MAP,
+            support_status=support.SupportStatus(version='7.0.0'),
+        ),
         ATTR_ATTRIBUTES: attributes.Schema(
             _("A map of resource names to the specified attribute of each "
               "individual resource. "
@@ -206,6 +213,11 @@ class ResourceGroup(stack_resource.StackResource):
             support_status=support.SupportStatus(version='2014.2'),
             type=attributes.Schema.MAP
         ),
+        REMOVED_RSRC_LIST: attributes.Schema(
+            _("A list of removed resource names."),
+            support_status=support.SupportStatus(version='7.0.0'),
+            type=attributes.Schema.LIST
+        ),
     }
 
     rolling_update_schema = {
@@ -284,6 +296,13 @@ class ResourceGroup(stack_resource.StackResource):
             msg = _("Failed to validate: %s") % six.text_type(ex)
             raise exception.StackValidationFailed(message=msg)
 
+    def _current_blacklist(self):
+        db_rsrc_names = self.data().get('name_blacklist')
+        if db_rsrc_names:
+            return db_rsrc_names.split(',')
+        else:
+            return []
+
     def _name_blacklist(self):
         """Resolve the remove_policies to names for removal."""
 
@@ -291,28 +310,23 @@ class ResourceGroup(stack_resource.StackResource):
 
         # To avoid reusing names after removal, we store a comma-separated
         # blacklist in the resource data
-        db_rsrc_names = self.data().get('name_blacklist')
-        if db_rsrc_names:
-            current_blacklist = db_rsrc_names.split(',')
-        else:
-            current_blacklist = []
+        current_blacklist = self._current_blacklist()
 
         # Now we iterate over the removal policies, and update the blacklist
         # with any additional names
         rsrc_names = set(current_blacklist)
 
-        if nested:
-            for r in self.properties[self.REMOVAL_POLICIES]:
-                if self.REMOVAL_RSRC_LIST in r:
-                    # Tolerate string or int list values
-                    for n in r[self.REMOVAL_RSRC_LIST]:
-                        str_n = six.text_type(n)
-                        if str_n in nested:
-                            rsrc_names.add(str_n)
-                            continue
-                        rsrc = nested.resource_by_refid(str_n)
-                        if rsrc:
-                            rsrc_names.add(rsrc.name)
+        for r in self.properties[self.REMOVAL_POLICIES]:
+            if self.REMOVAL_RSRC_LIST in r:
+                # Tolerate string or int list values
+                for n in r[self.REMOVAL_RSRC_LIST]:
+                    str_n = six.text_type(n)
+                    if not nested or str_n in nested:
+                        rsrc_names.add(str_n)
+                        continue
+                    rsrc = nested.resource_by_refid(str_n)
+                    if rsrc:
+                        rsrc_names.add(rsrc.name)
 
         # If the blacklist has changed, update the resource data
         if rsrc_names != set(current_blacklist):
@@ -344,7 +358,8 @@ class ResourceGroup(stack_resource.StackResource):
             max_batch_size = batch_create[self.MAX_BATCH_SIZE]
             pause_sec = batch_create[self.PAUSE_TIME]
             checkers = self._replace(0, max_batch_size, pause_sec)
-            checkers[0].start()
+            if checkers:
+                checkers[0].start()
             return checkers
         else:
             names = self._resource_names()
@@ -383,10 +398,13 @@ class ResourceGroup(stack_resource.StackResource):
                 return False
         return True
 
+    def res_def_changed(self, prop_diff):
+        return self.RESOURCE_DEF in prop_diff
+
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if tmpl_diff:
             # parse update policy
-            if rsrc_defn.UPDATE_POLICY in tmpl_diff:
+            if tmpl_diff.update_policy_changed():
                 up = json_snippet.update_policy(self.update_policy_schema,
                                                 self.context)
                 self.update_policy = up
@@ -394,7 +412,7 @@ class ResourceGroup(stack_resource.StackResource):
         checkers = []
         self.properties = json_snippet.properties(self.properties_schema,
                                                   self.context)
-        if prop_diff and self.RESOURCE_DEF in prop_diff:
+        if prop_diff and self.res_def_changed(prop_diff):
             updaters = self._try_rolling_update()
             if updaters:
                 checkers.extend(updaters)
@@ -417,6 +435,12 @@ class ResourceGroup(stack_resource.StackResource):
         if key == self.REFS:
             vals = [grouputils.get_rsrc_id(self, key, False, n) for n in names]
             return attributes.select_from_attribute(vals, path)
+        if key == self.REFS_MAP:
+            refs_map = {n: grouputils.get_rsrc_id(self, key, False, n)
+                        for n in names}
+            return refs_map
+        if key == self.REMOVED_RSRC_LIST:
+            return self._current_blacklist()
         if key == self.ATTR_ATTRIBUTES:
             if not path:
                 raise exception.InvalidTemplateAttribute(
@@ -430,12 +454,15 @@ class ResourceGroup(stack_resource.StackResource):
 
     def build_resource_definition(self, res_name, res_defn):
         res_def = copy.deepcopy(res_defn)
+
         props = res_def.get(self.RESOURCE_DEF_PROPERTIES)
         if props:
-            repl_props = self._handle_repl_val(res_name, props)
-            res_def[self.RESOURCE_DEF_PROPERTIES] = repl_props
-        return template.HOTemplate20130523.rsrc_defn_from_snippet(res_name,
-                                                                  res_def)
+            props = self._handle_repl_val(res_name, props)
+
+        res_type = res_def[self.RESOURCE_DEF_TYPE]
+        meta = res_def[self.RESOURCE_DEF_METADATA]
+
+        return rsrc_defn.ResourceDefinition(res_name, res_type, props, meta)
 
     def get_resource_def(self, include_all=False):
         """Returns the resource definition portion of the group.
@@ -451,9 +478,9 @@ class ResourceGroup(stack_resource.StackResource):
         # assigned. Pass in a custom resolver to the properties to not
         # error when a parameter does not have a user entered value.
         def ignore_param_resolve(snippet):
-            while isinstance(snippet, function.Function):
+            if isinstance(snippet, function.Function):
                 try:
-                    snippet = snippet.result()
+                    return snippet.result()
                 except exception.UserParameterMissing:
                     return None
 
@@ -555,11 +582,15 @@ class ResourceGroup(stack_resource.StackResource):
                                  policy[self.MAX_BATCH_SIZE],
                                  policy[self.PAUSE_TIME])
 
+    def _resolve_attribute(self, name):
+        if name == self.REMOVED_RSRC_LIST:
+            return self._current_blacklist()
+
     def _update_timeout(self, batch_cnt, pause_sec):
         total_pause_time = pause_sec * max(batch_cnt - 1, 0)
         if total_pause_time >= self.stack.timeout_secs():
-            msg = _('The current %s will result in stack update '
-                    'timeout.') % rsrc_defn.UPDATE_POLICY
+            msg = _('The current update policy will result in stack update '
+                    'timeout.')
             raise ValueError(msg)
         return self.stack.timeout_secs() - total_pause_time
 
