@@ -38,6 +38,7 @@ from heat.engine import translation
 from heat.rpc import api as rpc_api
 
 cfg.CONF.import_opt('default_software_config_transport', 'heat.common.config')
+cfg.CONF.import_opt('max_server_name_length', 'heat.common.config')
 
 LOG = logging.getLogger(__name__)
 
@@ -82,6 +83,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         BLOCK_DEVICE_MAPPING_DEVICE_NAME,
         BLOCK_DEVICE_MAPPING_VOLUME_ID,
         BLOCK_DEVICE_MAPPING_IMAGE_ID,
+        BLOCK_DEVICE_MAPPING_IMAGE,
         BLOCK_DEVICE_MAPPING_SNAPSHOT_ID,
         BLOCK_DEVICE_MAPPING_SWAP_SIZE,
         BLOCK_DEVICE_MAPPING_DEVICE_TYPE,
@@ -93,6 +95,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         'device_name',
         'volume_id',
         'image_id',
+        'image',
         'snapshot_id',
         'swap_size',
         'device_type',
@@ -214,6 +217,23 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                     BLOCK_DEVICE_MAPPING_IMAGE_ID: properties.Schema(
                         properties.Schema.STRING,
                         _('The ID of the image to create a volume from.'),
+                        support_status=support.SupportStatus(
+                            status=support.DEPRECATED,
+                            version='7.0.0',
+                            message=_('Use property %s.') %
+                                    BLOCK_DEVICE_MAPPING_IMAGE,
+                            previous_status=support.SupportStatus(
+                                version='5.0.0')
+                        ),
+                        constraints=[
+                            constraints.CustomConstraint('glance.image')
+                        ],
+                    ),
+                    BLOCK_DEVICE_MAPPING_IMAGE: properties.Schema(
+                        properties.Schema.STRING,
+                        _('The ID or name of the image '
+                          'to create a volume from.'),
+                        support_status=support.SupportStatus(version='7.0.0'),
                         constraints=[
                             constraints.CustomConstraint('glance.image')
                         ],
@@ -449,7 +469,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         USER_DATA_UPDATE_POLICY: properties.Schema(
             properties.Schema.STRING,
             _('Policy on how to apply a user_data update; either by '
-              'ignorning it or by replacing the entire server.'),
+              'ignoring it or by replacing the entire server.'),
             default='REPLACE',
             constraints=[
                 constraints.AllowedValues(['REPLACE', 'IGNORE']),
@@ -558,9 +578,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         ),
     }
 
-    # Server host name limit to 53 characters by due to typical default
-    # linux HOST_NAME_MAX of 64, minus the .novalocal appended to the name
-    physical_resource_name_limit = 53
+    physical_resource_name_limit = cfg.CONF.max_server_name_length
 
     default_client_name = 'nova'
 
@@ -571,18 +589,31 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             translation.TranslationRule(
                 props,
                 translation.TranslationRule.REPLACE,
-                source_path=[self.NETWORKS, self.NETWORK_ID],
+                translation_path=[self.NETWORKS, self.NETWORK_ID],
                 value_name=self.NETWORK_UUID),
             translation.TranslationRule(
                 props,
                 translation.TranslationRule.RESOLVE,
-                source_path=[self.FLAVOR],
+                translation_path=[self.FLAVOR],
                 client_plugin=self.client_plugin('nova'),
                 finder='find_flavor_by_name_or_id'),
             translation.TranslationRule(
                 props,
                 translation.TranslationRule.RESOLVE,
-                source_path=[self.IMAGE],
+                translation_path=[self.IMAGE],
+                client_plugin=self.client_plugin('glance'),
+                finder='find_image_by_name_or_id'),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.REPLACE,
+                translation_path=[self.BLOCK_DEVICE_MAPPING_V2,
+                                  self.BLOCK_DEVICE_MAPPING_IMAGE],
+                value_name=self.BLOCK_DEVICE_MAPPING_IMAGE_ID),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                translation_path=[self.BLOCK_DEVICE_MAPPING_V2,
+                                  self.BLOCK_DEVICE_MAPPING_IMAGE],
                 client_plugin=self.client_plugin('glance'),
                 finder='find_image_by_name_or_id'),
         ]
@@ -591,21 +622,21 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                 translation.TranslationRule(
                     props,
                     translation.TranslationRule.RESOLVE,
-                    source_path=[self.NETWORKS, self.NETWORK_ID],
+                    translation_path=[self.NETWORKS, self.NETWORK_ID],
                     client_plugin=self.client_plugin('neutron'),
                     finder='find_resourceid_by_name_or_id',
                     entity='network'),
                 translation.TranslationRule(
                     props,
                     translation.TranslationRule.RESOLVE,
-                    source_path=[self.NETWORKS, self.NETWORK_SUBNET],
+                    translation_path=[self.NETWORKS, self.NETWORK_SUBNET],
                     client_plugin=self.client_plugin('neutron'),
                     finder='find_resourceid_by_name_or_id',
                     entity='subnet'),
                 translation.TranslationRule(
                     props,
                     translation.TranslationRule.RESOLVE,
-                    source_path=[self.NETWORKS, self.NETWORK_PORT],
+                    translation_path=[self.NETWORKS, self.NETWORK_PORT],
                     client_plugin=self.client_plugin('neutron'),
                     finder='find_resourceid_by_name_or_id',
                     entity='port')])
@@ -614,7 +645,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                 translation.TranslationRule(
                     props,
                     translation.TranslationRule.RESOLVE,
-                    source_path=[self.NETWORKS, self.NETWORK_ID],
+                    translation_path=[self.NETWORKS, self.NETWORK_ID],
                     client_plugin=self.client_plugin('nova'),
                     finder='get_nova_network_id')])
         return rules
@@ -783,8 +814,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             self.SOFTWARE_CONFIG_TRANSPORT] == self.POLL_TEMP_URL
 
     def transport_zaqar_message(self, props):
-        return props.get(
-            self.SOFTWARE_CONFIG_TRANSPORT) == self.ZAQAR_MESSAGE
+        return props[
+            self.SOFTWARE_CONFIG_TRANSPORT] == self.ZAQAR_MESSAGE
 
     def get_software_config(self, ud_content):
         try:
@@ -946,9 +977,9 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                     'boot_index': 0,
                     'delete_on_termination': False,
                 }
-            elif mapping.get(cls.BLOCK_DEVICE_MAPPING_IMAGE_ID):
+            elif mapping.get(cls.BLOCK_DEVICE_MAPPING_IMAGE):
                 bmd_dict = {
-                    'uuid': mapping.get(cls.BLOCK_DEVICE_MAPPING_IMAGE_ID),
+                    'uuid': mapping.get(cls.BLOCK_DEVICE_MAPPING_IMAGE),
                     'source_type': 'image',
                     'destination_type': 'volume',
                     'boot_index': 0,
@@ -1054,8 +1085,13 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             return
         for res in six.itervalues(self.stack):
             if res.has_interface('OS::Neutron::Subnet'):
-                subnet_net = (res.properties.get(subnet.Subnet.NETWORK_ID)
-                              or res.properties.get(subnet.Subnet.NETWORK))
+                subnet_net = res.properties.get(subnet.Subnet.NETWORK)
+                # Be wary of the case where we do not know a subnet's
+                # network. If that's the case, be safe and add it as a
+                # dependency.
+                if not subnet_net:
+                    deps += (self, res)
+                    continue
                 for net in nets:
                     # worry about network_id because that could be the match
                     # assigned to the subnet as well and could have been
@@ -1065,6 +1101,11 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                     if net_id and net_id == subnet_net:
                         deps += (self, res)
                         break
+                    # If we don't know a given net_id right now, it's
+                    # plausible this subnet depends on it.
+                    if not net_id:
+                        deps += (self, res)
+                        break
 
     def _update_flavor(self, prop_diff):
         flavor = prop_diff[self.FLAVOR]
@@ -1081,6 +1122,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         image_update_policy = (
             prop_diff.get(self.IMAGE_UPDATE_POLICY) or
             self.properties[self.IMAGE_UPDATE_POLICY])
+
         instance_meta = prop_diff.get(self.METADATA,
                                       self.properties[self.METADATA])
 
@@ -1157,20 +1199,21 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             return ud_update_policy == 'REPLACE'
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        if 'Metadata' in tmpl_diff:
+        if tmpl_diff.metadata_changed():
             # If SOFTWARE_CONFIG user_data_format is enabled we require
             # the "deployments" and "os-collect-config" keys for Deployment
             # polling.  We can attempt to merge the occ data, but any
             # metadata update containing deployments will be discarded.
+            new_md = json_snippet.metadata()
             if self.user_data_software_config():
                 metadata = self.metadata_get(True) or {}
-                new_occ_md = tmpl_diff['Metadata'].get('os-collect-config', {})
+                new_occ_md = new_md.get('os-collect-config', {})
                 occ_md = metadata.get('os-collect-config', {})
                 occ_md.update(new_occ_md)
-                tmpl_diff['Metadata']['os-collect-config'] = occ_md
+                new_md['os-collect-config'] = occ_md
                 deployment_md = metadata.get('deployments', [])
-                tmpl_diff['Metadata']['deployments'] = deployment_md
-            self.metadata_set(tmpl_diff['Metadata'])
+                new_md['deployments'] = deployment_md
+            self.metadata_set(new_md)
 
         updaters = []
         server = None
@@ -1223,15 +1266,16 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                 self.context, sd['id'])
             self.rpc_client().delete_software_config(
                 self.context, sc['id'])
-        except Exception as e:
+        except Exception:
             # Updating the software config transport is on a best-effort
             # basis as any raised exception here would result in the resource
             # going into an ERROR state, which will be replaced on the next
             # stack update. This is not desirable for a server. The old
             # transport will continue to work, and the new transport may work
             # despite exceptions in the above block.
-            LOG.error(_LE('Error while updating software config transport'))
-            LOG.exception(e)
+            LOG.exception(
+                _LE('Error while updating software config transport')
+            )
 
     def check_update_complete(self, updaters):
         """Push all updaters to completion in list order."""
@@ -1303,7 +1347,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         for mapping in bdm_v2:
             volume_id = mapping.get(self.BLOCK_DEVICE_MAPPING_VOLUME_ID)
             snapshot_id = mapping.get(self.BLOCK_DEVICE_MAPPING_SNAPSHOT_ID)
-            image_id = mapping.get(self.BLOCK_DEVICE_MAPPING_IMAGE_ID)
+            image_id = mapping.get(self.BLOCK_DEVICE_MAPPING_IMAGE)
             swap_size = mapping.get(self.BLOCK_DEVICE_MAPPING_SWAP_SIZE)
 
             property_tuple = (volume_id, snapshot_id, image_id, swap_size)
@@ -1312,7 +1356,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                 raise exception.ResourcePropertyConflict(
                     self.BLOCK_DEVICE_MAPPING_VOLUME_ID,
                     self.BLOCK_DEVICE_MAPPING_SNAPSHOT_ID,
-                    self.BLOCK_DEVICE_MAPPING_IMAGE_ID,
+                    self.BLOCK_DEVICE_MAPPING_IMAGE,
                     self.BLOCK_DEVICE_MAPPING_SWAP_SIZE)
 
             if property_tuple.count(None) == 4:
@@ -1320,7 +1364,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                         'swap_size must be specified.')
                 raise exception.StackValidationFailed(message=msg)
 
-            if any((volume_id, snapshot_id, image_id)):
+            if any((volume_id is not None, snapshot_id is not None,
+                    image_id is not None)):
                 bootable_vol = True
 
         return bootable_vol
@@ -1375,7 +1420,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
         # make sure the image exists if specified.
         image = self.properties[self.IMAGE]
-        if not image and not bootable_vol:
+        if image is None and not bootable_vol:
             msg = _('Neither image nor bootable volume is specified for'
                     ' instance %s') % self.name
             raise exception.StackValidationFailed(message=msg)
@@ -1391,7 +1436,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         networks_with_port = False
         for network in networks:
             networks_with_port = (networks_with_port or
-                                  network.get(self.NETWORK_PORT))
+                                  network.get(self.NETWORK_PORT) is not None)
             self._validate_network(network)
 
         # retrieve provider's absolute limits if it will be needed
@@ -1400,7 +1445,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         if metadata is not None or personality:
             limits = self.client_plugin().absolute_limits()
 
-        # if 'security_groups' present for the server and explict 'port'
+        # if 'security_groups' present for the server and explicit 'port'
         # in one or more entries in 'networks', raise validation error
         if networks_with_port and self.properties[self.SECURITY_GROUPS]:
             raise exception.ResourcePropertyConflict(
