@@ -14,13 +14,13 @@
 import json
 import uuid
 
-from keystoneclient import access as ks_access
-from keystoneclient import auth as ks_auth
-from keystoneclient.auth.identity import access as ks_auth_access
-from keystoneclient.auth.identity import v3 as ks_auth_v3
-from keystoneclient.auth import token_endpoint as ks_token_endpoint
-import keystoneclient.exceptions as kc_exception
-from keystoneclient import session as ks_session
+from keystoneauth1 import access as ks_access
+from keystoneauth1 import exceptions as kc_exception
+from keystoneauth1.identity import access as ks_auth_access
+from keystoneauth1.identity import generic as ks_auth
+from keystoneauth1 import loading as ks_loading
+from keystoneauth1 import session as ks_session
+from keystoneauth1 import token_endpoint as ks_token_endpoint
 from keystoneclient.v3 import client as kc_v3
 from keystoneclient.v3 import domains as kc_v3_domains
 import mox
@@ -29,7 +29,7 @@ import six
 
 from heat.common import config
 from heat.common import exception
-from heat.common import heat_keystoneclient
+from heat.engine.clients.os.keystone import heat_keystoneclient
 from heat.tests import common
 from heat.tests import utils
 
@@ -49,16 +49,19 @@ class KeystoneClientTest(common.HeatTestCase):
         self.mock_ks_v3_client_domain_mngr = self.m.CreateMock(
             kc_v3_domains.DomainManager)
         self.m.StubOutWithMock(kc_v3, "Client")
-        self.m.StubOutWithMock(ks_auth_v3, 'Password')
+        self.m.StubOutWithMock(ks_auth, 'Password')
         self.m.StubOutWithMock(ks_token_endpoint, 'Token')
         self.m.StubOutWithMock(ks_auth_access, 'AccessInfoPlugin')
-        self.m.StubOutWithMock(ks_auth, 'load_from_conf_options')
+        self.m.StubOutWithMock(ks_loading, 'load_auth_from_conf_options')
 
         cfg.CONF.set_override('auth_uri', 'http://server.test:5000/v2.0',
-                              group='keystone_authtoken')
-        cfg.CONF.set_override('stack_user_domain_id', 'adomain123')
-        cfg.CONF.set_override('stack_domain_admin', 'adminuser123')
-        cfg.CONF.set_override('stack_domain_admin_password', 'adminsecret')
+                              group='keystone_authtoken', enforce_type=True)
+        cfg.CONF.set_override('stack_user_domain_id', 'adomain123',
+                              enforce_type=True)
+        cfg.CONF.set_override('stack_domain_admin', 'adminuser123',
+                              enforce_type=True)
+        cfg.CONF.set_override('stack_domain_admin_password', 'adminsecret',
+                              enforce_type=True)
 
         self.addCleanup(self.m.VerifyAll)
 
@@ -74,20 +77,21 @@ class KeystoneClientTest(common.HeatTestCase):
         else:
             a.AndRaise(kc_exception.Unauthorized)
 
-        m = ks_auth.load_from_conf_options(cfg.CONF, 'trustee', trust_id=None)
+        m = ks_loading.load_auth_from_conf_options(
+            cfg.CONF, 'trustee', trust_id=None)
         m.AndReturn(mock_ks_auth)
 
     def _stub_domain_admin_client(self, domain_id=None):
         mock_ks_auth = self.m.CreateMockAnything()
         mock_ks_auth.get_token(mox.IsA(ks_session.Session)).AndReturn('tok')
 
-        m = ks_auth_v3.Password(auth_url='http://server.test:5000/v3',
-                                password='adminsecret',
-                                domain_id='adomain123',
-                                domain_name=None,
-                                user_domain_id='adomain123',
-                                user_domain_name=None,
-                                username='adminuser123')
+        m = ks_auth.Password(auth_url='http://server.test:5000/v3',
+                             password='adminsecret',
+                             domain_id='adomain123',
+                             domain_name=None,
+                             user_domain_id='adomain123',
+                             user_domain_name=None,
+                             username='adminuser123')
         m.AndReturn(mock_ks_auth)
 
         n = kc_v3.Client(session=mox.IsA(ks_session.Session),
@@ -97,31 +101,36 @@ class KeystoneClientTest(common.HeatTestCase):
 
         self.mock_admin_client.domains = self.mock_ks_v3_client_domain_mngr
 
-    def _stubs_v3(self, method='token', trust_scoped=True,
-                  user_id=None, auth_ref=None, client=True, project_id=None,
-                  stub_trust_context=False):
+    def _stubs_auth(self, method='token', trust_scoped=True,
+                    user_id=None, auth_ref=None, client=True, project_id=None,
+                    stub_trust_context=False, version=3):
         mock_auth_ref = self.m.CreateMockAnything()
         mock_ks_auth = self.m.CreateMockAnything()
 
+        self.patchobject(mock_ks_auth, 'get_auth_ref')
         if method == 'token':
             p = ks_token_endpoint.Token(token='abcd1234',
                                         endpoint='http://server.test:5000/v3')
-        elif method == 'auth_ref':
+        elif method == 'auth_ref' and version == 3:
             p = ks_auth_access.AccessInfoPlugin(
-                auth_url='http://server.test:5000/v3',
-                auth_ref=mox.IsA(ks_access.AccessInfo))
+                auth_ref=mox.IsA(ks_access.AccessInfoV3),
+                auth_url='http://server.test:5000/v3')
+        elif method == 'auth_ref' and version == 2:
+            p = ks_auth_access.AccessInfoPlugin(
+                auth_ref=mox.IsA(ks_access.AccessInfoV2),
+                auth_url='http://server.test:5000/v3')
 
         elif method == 'password':
-            p = ks_auth_v3.Password(auth_url='http://server.test:5000/v3',
-                                    username='test_username',
-                                    password='password',
-                                    project_id=project_id or 'test_tenant_id',
-                                    user_domain_id='adomain123')
+            p = ks_auth.Password(auth_url='http://server.test:5000/v3',
+                                 username='test_username',
+                                 password='password',
+                                 project_id=project_id or 'test_tenant_id',
+                                 user_domain_id='adomain123')
 
         elif method == 'trust':
-            p = ks_auth.load_from_conf_options(cfg.CONF,
-                                               'trustee',
-                                               trust_id='atrust123')
+            p = ks_loading.load_auth_from_conf_options(cfg.CONF,
+                                                       'trustee',
+                                                       trust_id='atrust123')
 
             mock_auth_ref.user_id = user_id or 'trustor_user_id'
             mock_auth_ref.project_id = project_id or 'test_tenant_id'
@@ -132,7 +141,6 @@ class KeystoneClientTest(common.HeatTestCase):
 
         if client:
             c = kc_v3.Client(session=mox.IsA(ks_session.Session),
-                             auth=mock_ks_auth,
                              region_name=None)
             c.AndReturn(self.mock_ks_v3_client)
 
@@ -151,7 +159,7 @@ class KeystoneClientTest(common.HeatTestCase):
     def test_username_length(self):
         """Test that user names >64 characters are properly truncated."""
 
-        self._stubs_v3()
+        self._stubs_auth()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -187,7 +195,7 @@ class KeystoneClientTest(common.HeatTestCase):
     def test_create_stack_user_error_norole(self):
         """Test error path when no role is found."""
 
-        self._stubs_v3()
+        self._stubs_auth()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -214,6 +222,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test creating a stack domain user."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -246,7 +255,7 @@ class KeystoneClientTest(common.HeatTestCase):
         ctx.trust_id = None
 
         # mock keystone client functions
-        self._stubs_v3()
+        self._stubs_auth()
         self.mock_ks_v3_client.users = self.m.CreateMockAnything()
         mock_user = self.m.CreateMockAnything()
         mock_user.id = 'auser123'
@@ -271,6 +280,7 @@ class KeystoneClientTest(common.HeatTestCase):
     def test_create_stack_domain_user_error_norole(self):
         """Test creating a stack domain user, no role error path."""
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         self._stub_domain_admin_client(domain_id=None)
@@ -290,6 +300,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test deleting a stack domain user."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -321,7 +332,7 @@ class KeystoneClientTest(common.HeatTestCase):
         ctx.trust_id = None
 
         # mock keystone client functions
-        self._stubs_v3()
+        self._stubs_auth()
         self.mock_ks_v3_client.users = self.m.CreateMockAnything()
         self.mock_ks_v3_client.users.delete(user='user123').AndReturn(None)
         self.m.ReplayAll()
@@ -334,6 +345,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test deleting a stack domain user, wrong domain."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -356,6 +368,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test deleting a stack domain user, wrong project."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -378,7 +391,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test deleting a stack user."""
 
-        self._stubs_v3()
+        self._stubs_auth()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -399,7 +412,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test creating the client, token auth."""
 
-        self._stubs_v3()
+        self._stubs_auth()
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -416,7 +429,9 @@ class KeystoneClientTest(common.HeatTestCase):
 
         expected_auth_ref = {'token': {'id': 'ctx_token', 'expires': '123'},
                              'version': 'v2.0'}
-        self._stubs_v3(method='auth_ref', auth_ref=expected_auth_ref)
+        self._stubs_auth(method='auth_ref',
+                         auth_ref=expected_auth_ref,
+                         version=2)
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -438,7 +453,7 @@ class KeystoneClientTest(common.HeatTestCase):
                              'expires': '456',
                              'version': 'v3',
                              'methods': []}
-        self._stubs_v3(method='auth_ref', auth_ref=expected_auth_ref)
+        self._stubs_auth(method='auth_ref', auth_ref=expected_auth_ref)
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -455,7 +470,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test creating the client, password auth."""
 
-        self._stubs_v3(method='password')
+        self._stubs_auth(method='password')
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -476,16 +491,16 @@ class KeystoneClientTest(common.HeatTestCase):
         ctx.trust_id = None
         ctx.username = None
         ctx.password = None
-        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         self.assertRaises(exception.AuthorizationFailure,
-                          heat_ks_client._v3_client_init)
+                          heat_keystoneclient.KeystoneClient, ctx)
 
     def test_create_trust_context_trust_id(self):
 
         """Test create_trust_context with existing trust_id."""
 
-        self._stubs_v3(method='trust')
-        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        self._stubs_auth(method='trust')
+        cfg.CONF.set_override('deferred_auth_method', 'trusts',
+                              enforce_type=True)
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -511,13 +526,15 @@ class KeystoneClientTest(common.HeatTestCase):
             id = 'atrust123'
 
         self._stub_admin_auth()
-        mock_ks_auth, mock_auth_ref = self._stubs_v3(user_id='5678',
-                                                     project_id='42',
-                                                     stub_trust_context=True)
+        mock_ks_auth, mock_auth_ref = self._stubs_auth(user_id='5678',
+                                                       project_id='42',
+                                                       stub_trust_context=True)
 
-        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        cfg.CONF.set_override('deferred_auth_method', 'trusts',
+                              enforce_type=True)
         if delegate_roles:
-            cfg.CONF.set_override('trusts_delegated_roles', delegate_roles)
+            cfg.CONF.set_override('trusts_delegated_roles', delegate_roles,
+                                  enforce_type=True)
 
         trustor_roles = ['heat_stack_owner', 'admin', '__member__']
         trustee_roles = delegate_roles or trustor_roles
@@ -550,12 +567,14 @@ class KeystoneClientTest(common.HeatTestCase):
 
         self._stub_admin_auth()
 
-        mock_auth, mock_auth_ref = self._stubs_v3(user_id='5678',
-                                                  project_id='42',
-                                                  stub_trust_context=True)
+        mock_auth, mock_auth_ref = self._stubs_auth(user_id='5678',
+                                                    project_id='42',
+                                                    stub_trust_context=True)
 
-        cfg.CONF.set_override('deferred_auth_method', 'trusts')
-        cfg.CONF.set_override('trusts_delegated_roles', ['heat_stack_owner'])
+        cfg.CONF.set_override('deferred_auth_method', 'trusts',
+                              enforce_type=True)
+        cfg.CONF.set_override('trusts_delegated_roles', ['heat_stack_owner'],
+                              enforce_type=True)
 
         self.mock_ks_v3_client.trusts = self.m.CreateMockAnything()
         self.mock_ks_v3_client.trusts.create(
@@ -583,6 +602,7 @@ class KeystoneClientTest(common.HeatTestCase):
         cfg.CONF.clear_override('stack_domain_admin_password')
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.username = None
         ctx.password = None
         ctx.trust_id = None
@@ -607,8 +627,9 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test consuming a trust when initializing."""
 
-        self._stubs_v3(method='trust')
-        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        self._stubs_auth(method='trust')
+        cfg.CONF.set_override('deferred_auth_method', 'trusts',
+                              enforce_type=True)
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -625,8 +646,9 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test consuming a trust when initializing, error scoping."""
 
-        self._stubs_v3(method='trust', trust_scoped=False)
-        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        self._stubs_auth(method='trust', trust_scoped=False)
+        cfg.CONF.set_override('deferred_auth_method', 'trusts',
+                              enforce_type=True)
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -642,8 +664,9 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test consuming a trust when initializing, impersonation error."""
 
-        self._stubs_v3(method='trust', user_id='wrong_user_id')
-        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        self._stubs_auth(method='trust', user_id='wrong_user_id')
+        cfg.CONF.set_override('deferred_auth_method', 'trusts',
+                              enforce_type=True)
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -659,7 +682,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test trust_id is takes precedence username/password specified."""
 
-        self._stubs_v3(method='trust')
+        self._stubs_auth(method='trust')
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -673,7 +696,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test trust_id takes precedence when token specified."""
 
-        self._stubs_v3(method='trust')
+        self._stubs_auth(method='trust')
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -688,8 +711,9 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test delete_trust when deleting trust."""
 
-        self._stubs_v3()
-        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        self._stubs_auth()
+        cfg.CONF.set_override('deferred_auth_method', 'trusts',
+                              enforce_type=True)
         self.mock_ks_v3_client.trusts = self.m.CreateMockAnything()
         self.mock_ks_v3_client.trusts.delete('atrust123').AndReturn(None)
 
@@ -702,8 +726,9 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test delete_trust when trust already deleted."""
 
-        self._stubs_v3()
-        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        self._stubs_auth()
+        cfg.CONF.set_override('deferred_auth_method', 'trusts',
+                              enforce_type=True)
         self.mock_ks_v3_client.trusts = self.m.CreateMockAnything()
         self.mock_ks_v3_client.trusts.delete('atrust123').AndRaise(
             kc_exception.NotFound)
@@ -717,7 +742,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test disabling a stack user."""
 
-        self._stubs_v3()
+        self._stubs_auth()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -734,7 +759,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test enabling a stack user."""
 
-        self._stubs_v3()
+        self._stubs_auth()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -760,6 +785,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test enabling a stack domain user."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -781,7 +807,7 @@ class KeystoneClientTest(common.HeatTestCase):
         ctx.trust_id = None
 
         # mock keystone client functions
-        self._stubs_v3()
+        self._stubs_auth()
         self.mock_ks_v3_client.users = self.m.CreateMockAnything()
         self.mock_ks_v3_client.users.update(user='user123', enabled=True
                                             ).AndReturn(None)
@@ -795,6 +821,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test enabling a stack domain user, wrong project."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -810,6 +837,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test enabling a stack domain user, wrong domain."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -825,6 +853,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test disabling a stack domain user."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -846,7 +875,7 @@ class KeystoneClientTest(common.HeatTestCase):
         ctx.trust_id = None
 
         # mock keystone client functions
-        self._stubs_v3()
+        self._stubs_auth()
         self.mock_ks_v3_client.users = self.m.CreateMockAnything()
         self.mock_ks_v3_client.users.update(user='user123', enabled=False
                                             ).AndReturn(None)
@@ -860,6 +889,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test disabling a stack domain user, wrong project."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -875,6 +905,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test disabling a stack domain user, wrong domain."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -888,6 +919,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
     def test_delete_stack_domain_user_keypair(self):
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -918,7 +950,7 @@ class KeystoneClientTest(common.HeatTestCase):
         ctx.trust_id = None
 
         # mock keystone client functions
-        self._stubs_v3()
+        self._stubs_auth()
         self.mock_ks_v3_client.credentials = self.m.CreateMockAnything()
         self.mock_ks_v3_client.credentials.delete(
             'acredentialid').AndReturn(None)
@@ -931,6 +963,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
     def test_delete_stack_domain_user_keypair_error_project(self):
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -946,6 +979,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
     def test_delete_stack_domain_user_keypair_error_domain(self):
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         # mock keystone client functions
@@ -972,7 +1006,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         """Test creating ec2 credentials."""
 
-        self._stubs_v3()
+        self._stubs_auth()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -1010,6 +1044,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self._stub_domain_admin_client(domain_id=None)
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         ex_data = {'access': 'dummy_access2',
@@ -1044,7 +1079,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test creating ec2 credentials for domain user, fallback path."""
         self._clear_domain_override()
 
-        self._stubs_v3()
+        self._stubs_auth()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -1081,7 +1116,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test getting ec2 credential by id."""
 
         user_id = 'atestuser'
-        self._stubs_v3(user_id=user_id)
+        self._stubs_auth(user_id=user_id)
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -1131,7 +1166,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test getting ec2 credential by access."""
 
         user_id = 'atestuser'
-        self._stubs_v3(user_id=user_id)
+        self._stubs_auth(user_id=user_id)
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -1149,6 +1184,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test getting ec2 credential error path."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
@@ -1159,7 +1195,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test deleting ec2 credential by id."""
 
         user_id = 'atestuser'
-        self._stubs_v3(user_id=user_id)
+        self._stubs_auth(user_id=user_id)
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -1186,7 +1222,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test deleting ec2 credential by access."""
 
         user_id = 'atestuser'
-        self._stubs_v3(user_id=user_id)
+        self._stubs_auth(user_id=user_id)
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -1205,6 +1241,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test deleting ec2 credential error path."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
 
         self.m.ReplayAll()
@@ -1216,6 +1253,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test the create_stack_domain_project function."""
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
         expected_name = '%s-astack' % ctx.tenant_id
 
@@ -1239,7 +1277,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
-
+        self.patchobject(ctx, '_create_auth_plugin')
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         self.assertEqual(ctx.tenant_id,
                          heat_ks_client.create_stack_domain_project('astack'))
@@ -1259,6 +1297,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         heat_ks_client.delete_stack_domain_project(project_id='aprojectid')
@@ -1274,6 +1313,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         heat_ks_client.delete_stack_domain_project(project_id='aprojectid')
@@ -1289,6 +1329,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         heat_ks_client.delete_stack_domain_project(project_id='aprojectid')
@@ -1307,6 +1348,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         heat_ks_client.delete_stack_domain_project(project_id='aprojectid')
@@ -1318,33 +1360,33 @@ class KeystoneClientTest(common.HeatTestCase):
         self._clear_domain_override()
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         heat_ks_client.delete_stack_domain_project(project_id='aprojectid')
 
     def _stub_domain_user_pw_auth(self):
-        ks_auth_v3.Password(auth_url='http://server.test:5000/v3',
-                            user_id='duser',
-                            password='apassw',
-                            project_id='aproject',
-                            user_domain_id='adomain123').AndReturn('dummyauth')
+        ks_auth.Password(auth_url='http://server.test:5000/v3',
+                         user_id='duser',
+                         password='apassw',
+                         project_id='aproject',
+                         user_domain_id='adomain123').AndReturn('dummyauth')
 
     def test_stack_domain_user_token(self):
         """Test stack_domain_user_token function."""
         dum_tok = 'dummytoken'
+        ctx = utils.dummy_context()
         mock_ks_auth = self.m.CreateMockAnything()
         mock_ks_auth.get_token(mox.IsA(ks_session.Session)).AndReturn(dum_tok)
-
-        m = ks_auth_v3.Password(auth_url='http://server.test:5000/v3',
-                                password='apassw',
-                                project_id='aproject',
-                                user_id='duser',
-                                include_catalog=False)
+        self.patchobject(ctx, '_create_auth_plugin')
+        m = ks_auth.Password(auth_url='http://server.test:5000/v3',
+                             password='apassw',
+                             project_id='aproject',
+                             user_id='duser')
         m.AndReturn(mock_ks_auth)
 
         self.m.ReplayAll()
 
-        ctx = utils.dummy_context()
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         token = heat_ks_client.stack_domain_user_token(user_id='duser',
@@ -1356,6 +1398,8 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test stack_domain_user_token error path."""
         self._clear_domain_override()
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
+
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         self.assertRaises(exception.Error,
@@ -1369,6 +1413,7 @@ class KeystoneClientTest(common.HeatTestCase):
         self._clear_domain_override()
 
         ctx = utils.dummy_context()
+        self.patchobject(ctx, '_create_auth_plugin')
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         self.assertIsNone(heat_ks_client.delete_stack_domain_project(
@@ -1376,7 +1421,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
     def _test_url_for(self, service_url, expected_kwargs, ctx=None, **kwargs):
         """Testing url_for depending on different ways to pass region name."""
-        mock_ks_auth, mock_auth_ref = self._stubs_v3(client=False)
+        mock_ks_auth, mock_auth_ref = self._stubs_auth(client=False)
         mock_ks_auth.get_endpoint(mox.IsA(ks_session.Session),
                                   **expected_kwargs).AndReturn(service_url)
 
@@ -1392,7 +1437,8 @@ class KeystoneClientTest(common.HeatTestCase):
         Test that None value is passed as region name if region name is not
         specified in the config file or as one of the arguments.
         """
-        cfg.CONF.set_override('region_name_for_services', None)
+        cfg.CONF.set_override('region_name_for_services', None,
+                              enforce_type=True)
         service_url = 'http://example.com:1234/v1'
         kwargs = {
             'region_name': None
@@ -1405,7 +1451,8 @@ class KeystoneClientTest(common.HeatTestCase):
         Test that region name passed as argument is not override by region name
         specified in the config file.
         """
-        cfg.CONF.set_override('region_name_for_services', 'RegionOne')
+        cfg.CONF.set_override('region_name_for_services', 'RegionOne',
+                              enforce_type=True)
         service_url = 'http://regiontwo.example.com:1234/v1'
         kwargs = {
             'region_name': 'RegionTwo'
@@ -1420,7 +1467,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """
         region_name_for_services = 'RegionOne'
         cfg.CONF.set_override('region_name_for_services',
-                              region_name_for_services)
+                              region_name_for_services, enforce_type=True)
         kwargs = {
             'region_name': region_name_for_services
         }
@@ -1433,7 +1480,8 @@ class KeystoneClientTest(common.HeatTestCase):
         Test that default region name for services from context is passed
         if region name is not specified in arguments.
         """
-        cfg.CONF.set_override('region_name_for_services', 'RegionOne')
+        cfg.CONF.set_override('region_name_for_services', 'RegionOne',
+                              enforce_type=True)
         service_url = 'http://regiontwo.example.com:1234/v1'
         region_name_for_services = 'RegionTwo'
         expected_kwargs = {
@@ -1451,7 +1499,8 @@ class KeystoneClientTest(common.HeatTestCase):
 
 class KeystoneClientTestDomainName(KeystoneClientTest):
     def setUp(self):
-        cfg.CONF.set_override('stack_user_domain_name', 'fake_domain_name')
+        cfg.CONF.set_override('stack_user_domain_name', 'fake_domain_name',
+                              enforce_type=True)
         super(KeystoneClientTestDomainName, self).setUp()
         cfg.CONF.clear_override('stack_user_domain_id')
 
@@ -1471,15 +1520,16 @@ class KeystoneClientTestDomainName(KeystoneClientTest):
         if domain_id:
             a = self.m.CreateMockAnything()
             a.domain_id = domain_id
-            mock_ks_auth.get_access(mox.IsA(ks_session.Session)).AndReturn(a)
+            mock_ks_auth.get_access(
+                mox.IsA(ks_session.Session)).AndReturn(a)
 
-        m = ks_auth_v3.Password(auth_url='http://server.test:5000/v3',
-                                password='adminsecret',
-                                domain_id=None,
-                                domain_name='fake_domain_name',
-                                user_domain_id=None,
-                                user_domain_name='fake_domain_name',
-                                username='adminuser123')
+        m = ks_auth.Password(auth_url='http://server.test:5000/v3',
+                             password='adminsecret',
+                             domain_id=None,
+                             domain_name='fake_domain_name',
+                             user_domain_id=None,
+                             user_domain_name='fake_domain_name',
+                             username='adminuser123')
 
         m.AndReturn(mock_ks_auth)
 
@@ -1491,12 +1541,12 @@ class KeystoneClientTestDomainName(KeystoneClientTest):
         self.mock_admin_client.domains = self.mock_ks_v3_client_domain_mngr
 
     def _stub_domain_user_pw_auth(self):
-        ks_auth_v3.Password(auth_url='http://server.test:5000/v3',
-                            user_id='duser',
-                            password='apassw',
-                            project_id='aproject',
-                            user_domain_name='fake_domain_name'
-                            ).AndReturn('dummyauth')
+        ks_auth.Password(auth_url='http://server.test:5000/v3',
+                         user_id='duser',
+                         password='apassw',
+                         project_id='aproject',
+                         user_domain_name='fake_domain_name'
+                         ).AndReturn('dummyauth')
 
     def test_enable_stack_domain_user_error_project(self):
         p = super(KeystoneClientTestDomainName, self)
