@@ -28,6 +28,7 @@ from heat.engine.clients.os import swift
 from heat.engine.clients.os import zaqar
 from heat.engine import service
 from heat.engine import service_software_config
+from heat.engine import software_config_io as swc_io
 from heat.objects import resource as resource_objects
 from heat.objects import software_config as software_config_object
 from heat.objects import software_deployment as software_deployment_object
@@ -148,8 +149,11 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         self.assertEqual(kwargs['group'], config['group'])
         self.assertEqual(kwargs['name'], config['name'])
         self.assertEqual(kwargs['config'], config['config'])
-        self.assertEqual(kwargs['inputs'], config['inputs'])
-        self.assertEqual(kwargs['outputs'], config['outputs'])
+        self.assertEqual([{'name': 'mode', 'type': 'String'}],
+                         config['inputs'])
+        self.assertEqual([{'name': 'endpoint', 'type': 'String',
+                           'error_output': False}],
+                         config['outputs'])
         self.assertEqual(kwargs['options'], config['options'])
 
     def test_delete_software_config(self):
@@ -311,6 +315,20 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
             deployment,
             self.engine.show_software_deployment(self.ctx, deployment_id))
 
+    def test_check_software_deployment(self):
+        deployment_id = str(uuid.uuid4())
+        ex = self.assertRaises(dispatcher.ExpectedException,
+                               self.engine.check_software_deployment,
+                               self.ctx, deployment_id, 10)
+        self.assertEqual(exception.NotFound, ex.exc_info[0])
+
+        deployment = self._create_software_deployment()
+        self.assertIsNotNone(deployment)
+        deployment_id = deployment['id']
+        self.assertEqual(
+            deployment,
+            self.engine.check_software_deployment(self.ctx, deployment_id, 10))
+
     @mock.patch.object(service_software_config.SoftwareConfigService,
                        '_push_metadata_software_deployments')
     def test_signal_software_deployment(self, pmsd):
@@ -336,7 +354,7 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         deployment_id = deployment['id']
         res = self.engine.signal_software_deployment(
             self.ctx, deployment_id, {}, None)
-        self.assertEqual('deployment succeeded', res)
+        self.assertEqual('deployment %s succeeded' % deployment_id, res)
 
         sd = software_deployment_object.SoftwareDeployment.get_by_id(
             self.ctx, deployment_id)
@@ -359,7 +377,7 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
             deployment_id,
             {'foo': 'bar', 'deploy_status_code': 0},
             None)
-        self.assertEqual('deployment succeeded', result)
+        self.assertEqual('deployment %s succeeded' % deployment_id, result)
         sd = software_deployment_object.SoftwareDeployment.get_by_id(
             self.ctx, deployment_id)
         self.assertEqual('COMPLETE', sd.status)
@@ -386,7 +404,7 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
                 'deploy_stderr': 'Its gone Pete Tong'
             },
             None)
-        self.assertEqual('deployment failed (-1)', result)
+        self.assertEqual('deployment %s failed (-1)' % deployment_id, result)
         sd = software_deployment_object.SoftwareDeployment.get_by_id(
             self.ctx, deployment_id)
         self.assertEqual('FAILED', sd.status)
@@ -417,7 +435,7 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
                 'deploy_stderr': 'Its gone Pete Tong'
             },
             None)
-        self.assertEqual('deployment failed', result)
+        self.assertEqual('deployment %s failed' % deployment_id, result)
 
         sd = software_deployment_object.SoftwareDeployment.get_by_id(
             self.ctx, deployment_id)
@@ -896,3 +914,116 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         zaqar_client.queue.assert_called_once_with('6789')
         queue.pop.assert_called_once_with()
         ssd.assert_called_once_with(self.ctx, deployment_id, 'ok', None)
+
+
+class SoftwareConfigIOSchemaTest(common.HeatTestCase):
+    def test_input_config_empty(self):
+        name = 'foo'
+        inp = swc_io.InputConfig(name=name)
+        self.assertIsNone(inp.default())
+        self.assertIs(False, inp.replace_on_change())
+        self.assertEqual(name, inp.name())
+        self.assertEqual({'name': name, 'type': 'String'}, inp.as_dict())
+        self.assertEqual((name, None), inp.input_data())
+
+    def test_input_config(self):
+        name = 'bar'
+        inp = swc_io.InputConfig(name=name, description='test', type='Number',
+                                 default=0, replace_on_change=True)
+        self.assertEqual('0', inp.default())
+        self.assertIs(True, inp.replace_on_change())
+        self.assertEqual(name, inp.name())
+        self.assertEqual({'name': name, 'type': 'Number',
+                          'description': 'test', 'default': '0',
+                          'replace_on_change': True},
+                         inp.as_dict())
+        self.assertEqual((name, None), inp.input_data())
+
+    def test_input_config_value(self):
+        name = 'baz'
+        inp = swc_io.InputConfig(name=name, type='Number',
+                                 default=0, value=42)
+        self.assertEqual('0', inp.default())
+        self.assertIs(False, inp.replace_on_change())
+        self.assertEqual(name, inp.name())
+        self.assertEqual({'name': name, 'type': 'Number',
+                          'default': '0', 'value': 42},
+                         inp.as_dict())
+        self.assertEqual((name, 42), inp.input_data())
+
+    def test_input_config_no_name(self):
+        self.assertRaises(ValueError, swc_io.InputConfig, type='String')
+
+    def test_input_config_extra_key(self):
+        self.assertRaises(ValueError, swc_io.InputConfig,
+                          name='test', bogus='wat')
+
+    def test_input_types(self):
+        swc_io.InputConfig(name='str', type='String').as_dict()
+        swc_io.InputConfig(name='num', type='Number').as_dict()
+        swc_io.InputConfig(name='list', type='CommaDelimitedList').as_dict()
+        swc_io.InputConfig(name='json', type='Json').as_dict()
+        swc_io.InputConfig(name='bool', type='Boolean').as_dict()
+
+        self.assertRaises(ValueError, swc_io.InputConfig,
+                          name='bogus', type='BogusType')
+
+    def test_output_config_empty(self):
+        name = 'foo'
+        outp = swc_io.OutputConfig(name=name)
+        self.assertEqual(name, outp.name())
+        self.assertEqual({'name': name, 'type': 'String',
+                          'error_output': False},
+                         outp.as_dict())
+
+    def test_output_config(self):
+        name = 'bar'
+        outp = swc_io.OutputConfig(name=name, description='test',
+                                   type='Json', error_output=True)
+        self.assertEqual(name, outp.name())
+        self.assertIs(True, outp.error_output())
+        self.assertEqual({'name': name, 'type': 'Json',
+                          'description': 'test', 'error_output': True},
+                         outp.as_dict())
+
+    def test_output_config_no_name(self):
+        self.assertRaises(ValueError, swc_io.OutputConfig, type='String')
+
+    def test_output_config_extra_key(self):
+        self.assertRaises(ValueError, swc_io.OutputConfig,
+                          name='test', bogus='wat')
+
+    def test_output_types(self):
+        swc_io.OutputConfig(name='str', type='String').as_dict()
+        swc_io.OutputConfig(name='num', type='Number').as_dict()
+        swc_io.OutputConfig(name='list', type='CommaDelimitedList').as_dict()
+        swc_io.OutputConfig(name='json', type='Json').as_dict()
+        swc_io.OutputConfig(name='bool', type='Boolean').as_dict()
+
+        self.assertRaises(ValueError, swc_io.OutputConfig,
+                          name='bogus', type='BogusType')
+
+    def test_check_io_schema_empty_list(self):
+        swc_io.check_io_schema_list([])
+
+    def test_check_io_schema_string(self):
+        self.assertRaises(TypeError, swc_io.check_io_schema_list, '')
+
+    def test_check_io_schema_dict(self):
+        self.assertRaises(TypeError, swc_io.check_io_schema_list, {})
+
+    def test_check_io_schema_list_dict(self):
+        swc_io.check_io_schema_list([{'name': 'foo'}])
+
+    def test_check_io_schema_list_string(self):
+        self.assertRaises(TypeError, swc_io.check_io_schema_list, ['foo'])
+
+    def test_check_io_schema_list_list(self):
+        self.assertRaises(TypeError, swc_io.check_io_schema_list, [['foo']])
+
+    def test_check_io_schema_list_none(self):
+        self.assertRaises(TypeError, swc_io.check_io_schema_list, [None])
+
+    def test_check_io_schema_list_mixed(self):
+        self.assertRaises(TypeError, swc_io.check_io_schema_list,
+                          [{'name': 'foo'}, ('name', 'bar')])
