@@ -104,6 +104,8 @@ class OSDBInstanceTest(common.HeatTestCase):
         self.nova = self.m.CreateMockAnything()
         self.m.StubOutWithMock(trove.TroveClientPlugin, '_create')
         self.stub_TroveFlavorConstraint_validate()
+        self.patchobject(resource.Resource, 'is_using_neutron',
+                         return_value=True)
 
     def _setup_test_clouddbinstance(self, name, t):
         stack_name = '%s_stack' % name
@@ -467,6 +469,19 @@ class OSDBInstanceTest(common.HeatTestCase):
         self.assertRaises(exception.StackValidationFailed, instance.validate)
         self.m.VerifyAll()
 
+    def test_osdatabase_prop_validation_db_name_hyphens(self):
+        t = template_format.parse(db_template)
+        t['Resources']['MySqlCloudDB']['Properties']['databases'] = [
+            {"name": "-foo-bar-"}]
+        t['Resources']['MySqlCloudDB']['Properties']['users'] = [
+            {"name": "testuser",
+             "password": "pass",
+             "databases": ["-foo-bar-"]}]
+        instance = self._setup_test_clouddbinstance('dbinstance_test', t)
+        self._stubout_validate(instance)
+        self.assertIsNone(instance.validate())
+        self.m.VerifyAll()
+
     def test_osdatabase_prop_validation_users_none(self):
         t = template_format.parse(db_template)
         t['Resources']['MySqlCloudDB']['Properties']['users'] = []
