@@ -13,15 +13,18 @@
 
 import argparse
 import re
-
-import six
 import sys
 
+from oslo_log import log
+import six
+
 from heat.common.i18n import _
+from heat.common.i18n import _LW
 from heat.engine import constraints
 from heat.engine import resources
 from heat.engine import support
 
+LOG = log.getLogger(__name__)
 
 class HeatCustomGuidelines(object):
 
@@ -33,12 +36,15 @@ class HeatCustomGuidelines(object):
         global_env = resources.global_env()
         for resource_type in global_env.get_types():
             cls = global_env.get_class(resource_type)
+            module = cls.__module__
             # Skip resources, which defined as template resource in environment
-            if cls.__module__ == 'heat.engine.resources.template_resource':
+            if module == 'heat.engine.resources.template_resource':
                 continue
-            if (lambda module: True
-                if [path for path in exclude if path in module]
-                    else False)(cls.__module__.replace('.', '/')):
+            # Skip discovered plugin resources
+            if module == 'heat.engine.plugins':
+                continue
+            path = module.replace('.', '/')
+            if any(path.startswith(excl_path) for excl_path in exclude):
                 continue
             self.resources_classes.append(cls)
 
@@ -146,7 +152,12 @@ class HeatCustomGuidelines(object):
 
     def check_trailing_spaces(self):
         for cls in self.resources_classes:
-            cls_file = open(cls.__module__.replace('.', '/') + '.py')
+            try:
+                cls_file = open(cls.__module__.replace('.', '/') + '.py')
+            except IOError as ex:
+                LOG.warning(_LW('Cannot perform trailing spaces check on '
+                                'resource module: %s') % six.text_type(ex))
+                continue
             lines = [line.strip() for line in cls_file.readlines()]
             idx = 0
             kwargs = {'path': cls.__module__}
